/** DMControl.java
 *  
 *  This object implements the Control part of the MVC framework.
 *  Takes care of handling events (keys + mouse + mouseMotion).
 *
 *  Enrique Campos-Nanez.
 *  ec3z@virginia.edu
 *  @version 1.0a
 */

import java.awt.*;
import java.awt.event.*;
import DMControl;
import DMModel;

public class DMControl extends KeyAdapter
    implements MouseListener, MouseMotionListener
{
    DMView V;
    DMModel M;
    DMObject startO;
    int action;
    boolean editing;
    Point start, end;

    static final int NONE = 0;
    static final int MOVING = 1;
    static final int SELECTING = 2;
    static final int LINKING = 3;

    /** The only constructor required the view object as a reference.
     *  as well as the model.
     *  @param inV the view (DMView) object.
     *  @param inM the model (DMModel) object.
     */
    public DMControl( DMView inV, DMModel inM )
    {
	// Store the references
	V = inV;
	M = inM;
	editing = false;
	action = DMControl.NONE;
    }

    public DMObject modelIsHit( Dimension r, Point p )
    {
	int i;
	for( i=0; i<M.getSize(); i++) {
	    if (M.getObjectAt(i).isHit(r,p))
		return M.getObjectAt(i);
	}
	return null;
    }

    public int getSelectedCount()
    {
	int i, count;
	
	count = 0;
	for (i=0; i<M.getSize(); i++)
	    if (M.getObjectAt(i).isSelected())
		count++;
	return count;
    }

    public DMObject getIthSelected( int index )
    {
	int i, count;
	
	count = 1;

	for (i=0; i<M.getSize(); i++)
	    if (M.getObjectAt(i).isSelected()) 
		if (count == index) 
		    return M.getObjectAt(i);
		else
		    count++;
	// Not found
	return null;
    }

    public void unselectAll()
    {
	int i;

	for (i=0; i<M.getSize(); i++) 
	    M.getObjectAt(i).unselect();
    }

    public void mouseEntered( MouseEvent me ) {}

    public void mouseExited( MouseEvent me ) {}

    public void mouseClicked( MouseEvent me )
    {
	DMObject e;
	boolean shift = me.isShiftDown();

	switch( M . getEditionMode() ) {

	case DMModel.SELECT_MODE:
	    e = modelIsHit( V. getBounds(). getSize(), me.getPoint());
	    if ( e != null ) {
		if (shift) {
		    e . toggleSelect();
		} else {
		    unselectAll();
		    e . select();
		}
	    } else {
		if (!shift) 
		    unselectAll();
	    }
	    M . updateAllViews();
	    break;

	case DMModel.ICON_MODE:
	    M . addSimpleObject( V. getBounds(). getSize(), me.getPoint(), true );
	    M . updateAllViews();
	    break;
	}
    }

    public void mousePressed( MouseEvent me )
    {
	DMObject e;
	boolean shift = me.isShiftDown();

	V.requestFocus();
	// Check if someone is hit
	e = modelIsHit(  V. getBounds(). getSize(), me.getPoint() );

	switch (M .getEditionMode()) {

	case DMModel.SELECT_MODE:
	    if (e != null && e.isSelected()) { // We have a hit
		action = DMControl.MOVING;
		start = me.getPoint();
	    } else {
		if (!shift)
		    unselectAll();
		action = DMControl.SELECTING;
		start = me.getPoint();
		end = start;
	    }
	    break;

	case DMModel.RELATION_MODE:
	    if (e != null ) { // we have a hit
		action = DMControl.LINKING;
		start = me.getPoint();
		end = start;
		startO = e;
	    } else
		action = DMControl.NONE;
	    break;
	}

    }

    public void drawControls( Graphics g )
    {
	int x1=0, x2=0, y1=0, y2=0;

	switch(action) {
	case (DMControl.SELECTING):
	    if (start.getX()<end.getX()) {
		x1 = (int) start.getX();
		x2 = (int) end.getX();
	    } else {
		x2 = (int) start.getX();
		x1 = (int)end.getX();
	    }
	    if (start.getY()<end.getY()) {
		y1 = (int) start.getY();
		y2 = (int) end.getY();
	    } else {
		y2 = (int) start.getY();
		y1 = (int) end.getY();
	    }
	    break;
	case (DMControl.LINKING):
	    x1 = (int)start.getX();
	    y1 = (int)start.getY();
	    x2 = (int)end.getX();
	    y2 = (int)end.getY();
	}
	
	switch (action) {
	case (DMControl.SELECTING):
	    g.drawRect(x1,y1,x2-x1,y2-y1);
	    break;
	case (DMControl.LINKING):
	    g.drawLine(x1,y1,x2,y2);
	    break;
	}
    }

    public void mouseReleased( MouseEvent me )
    {
		DMObject e = modelIsHit( V.getBounds().getSize(), me.getPoint());
		int i;
		boolean shift = me.isShiftDown();
	
		switch (action) {
			
		case (DMControl.SELECTING):
			for ( i=0; i<M.getSize(); i++) {
			if (M.getObjectAt(i).isContained(V.getBounds().getSize(),start,me.getPoint()))
				if (shift)
				M.getObjectAt(i).toggleSelect();
				else
				M.getObjectAt(i).select();
			}
			M.updateAllViews();
			break;
	
		case (DMControl.MOVING):
			Dimension r = new Dimension( (int) (me.getPoint().getX()-start.getX()), 
						 (int) (me.getPoint().getY()-start.getY()));
			for ( i=0; i<M.getSize(); i++ ) {
				if (M.getObjectAt(i).isSelected())
					M.getObjectAt(i).moveBy(V.getBounds().getSize(),r);
				M.setDirty(true);
			}
			M.updateAllViews();
			
			break;
		   
		case (DMControl.LINKING):
			if (action == DMControl.LINKING) {
			if (e!=null) {
				M.addRelationObject(startO,e,true);
				M.updateAllViews();
			} else {
				V. repaint();
			}
			}
		}
		action = DMControl.NONE;
    }

    public void mouseMoved( MouseEvent me )
    {
    }

    public void mouseDragged( MouseEvent me )
    {
	Dimension r;
	int i;

	if (action == DMControl.MOVING) {
	    r = new Dimension( (int) (me.getPoint().getX()-start.getX()), 
					 (int) (me.getPoint().getY()-start.getY()));
	    for ( i=0; i<M.getSize(); i++ ) {
		if (M.getObjectAt(i).isSelected()) {
		    M.getObjectAt(i).moveBy(V.getBounds().getSize(),r);
		}
	    }
	    M.updateAllViews();
	    start = me.getPoint();
	} else {  
	    end = me.getPoint();
	    V.repaint();
	}
	
    }

    /* Implementation of selected key events.
     */
    public void keyTyped( KeyEvent e )
    {
		char c = e.getKeyChar();
		if (c=='\b') {
			M.removeSelected();
		}   
    }

}













